/*************************************************************************\
**  source       * cli0lcli.h
**  directory    * \SOLID\O32\CLI
**  description  * Light Client functions.
**               * 
**               * (C) Copyright Solid Information Technology Ltd 1997
\*************************************************************************/

#ifndef CLI0LCLI_H
#define CLI0LCLI_H

#undef SS_EXPORT

#if defined(_MSC_VER)
    /* Windows NT or WIN32S */
#   define SS_EXPORT __stdcall

#else 

    /* Other systems (e.g. Unix) */
#   define SS_EXPORT 
#endif

#if !defined(SS_LONG_BIT)
#  if defined(__GNUC__) && defined(__alpha__) /* (probably Linux/Alpha) */
#    define SS_LONG_BIT 64
#  endif /* __GNUC__ && __alpha__ */
#endif /* !SS_LONG_BIT */

#if !defined(SS_LONG_BIT)
#define SS_LONG_BIT 32
#endif /* !SS_LONG_BIT */

/* if ODBCVER is not defined, assume version 2.1 */
#ifndef ODBCVER
#define ODBCVER	0x0210
#endif

/* API declaration data types */
#define UCHAR   unsigned char           /*  8 bits */
#define SCHAR   signed char             /*  8 bits */
#define SDWORD  long int                /* 32 bits */
#define SWORD   short int               /* 16 bits */
#define UDWORD  unsigned long int       /* 32 bits */
#define UWORD   unsigned short int      /* 16 bits */
#define SDOUBLE double                  /* 64 bits */
#define SFLOAT  float                   /* 32 bits */
#define PTR     void*               
#define PTRSIZEDUINT unsigned long int

#if SS_LONG_BIT == 64
   /* 64-bit architecture */
#  undef SDWORD
#  undef UDWORD
#  define SDWORD  int                /* 32 bits */
#  define UDWORD  unsigned int       /* 32 bits */
#endif /* 64 bit long */

/* function return type */
typedef SWORD               RETCODE;

/* generic data structures */
typedef struct cli_stmt_st* HSTMT;
typedef struct cli_dbc_st*  HDBC;
typedef struct cli_env_st*  HENV;

/* return values for functions */
#define SQL_INVALID_HANDLE          (-2)
#define SQL_ERROR                   (-1)
#define SQL_SUCCESS                 0
#define SQL_SUCCESS_WITH_INFO       1
#define SQL_STILL_EXECUTING         2
#define SQL_NEED_DATA               99
#define SQL_NO_DATA_FOUND           100

/* flags for SQLFreeStmt */
#define SQL_CLOSE                   0
#define SQL_DROP                    1
#define SQL_UNBIND                  2
#define SQL_RESET_PARAMS            3

/* Special length values */
#define SQL_NULL_DATA               (-1)
#define SQL_DATA_AT_EXEC            (-2)
#define SQL_NTS                     (-3)

/* SQL type codes, as defined in ANSI SQL 2 (Section 17.1, Table 18)*/
#define SQL_CHAR            1  
#define SQL_NUMERIC         2  
#define SQL_DECIMAL         3
#define SQL_INTEGER         4  
#define SQL_SMALLINT        5
#define SQL_FLOAT           6  
#define SQL_REAL            7
#define SQL_DOUBLE          8
#define SQL_VARCHAR         12

#define SQL_DATE            9
#define SQL_TIME            10
#define SQL_TIMESTAMP       11

/* SQL extended datatypes < 0 */
#define SQL_LONGVARCHAR     (-1)
#define SQL_BINARY          (-2)
#define SQL_VARBINARY       (-3)
#define SQL_LONGVARBINARY   (-4)
#define SQL_BIGINT          (-5)
#define SQL_TINYINT         (-6)
#define SQL_BIT             (-7)

/* WIde versions are not supported by Light Client */
#define SQL_WCHAR		 	(-8)
#define SQL_WVARCHAR	 	(-9)
#define SQL_WLONGVARCHAR 	(-10)
#define SQL_C_WCHAR			SQL_WCHAR

#define SQL_TYPE_MAX        SQL_VARCHAR
#define SQL_TYPE_MIN        SQL_WLONGVARCHAR
#define SQL_ALL_TYPES       0

/* C type codes for the data types used in input and output of data
from the database */
#if (ODBCVER >= 0x0200)
#define SQL_SIGNED_OFFSET   (-20)
#define SQL_UNSIGNED_OFFSET (-22)
#endif  /* ODBCVER >= 0x0200 */

#define SQL_C_DATE      SQL_DATE
#define SQL_C_TIME      SQL_TIME
#define SQL_C_TIMESTAMP SQL_TIMESTAMP
#define SQL_C_BINARY    SQL_BINARY
#define SQL_C_BIT       SQL_BIT
#define SQL_C_TINYINT   SQL_TINYINT
#define SQL_C_CHAR      SQL_CHAR         /* CHAR[] */
#define SQL_C_LONG      SQL_INTEGER      /* SDWORD */
#define SQL_C_SHORT     SQL_SMALLINT     /* SWORD  */
#define SQL_C_FLOAT     SQL_REAL         /* SFLOAT */
#define SQL_C_DOUBLE    SQL_DOUBLE       /* SDOUBLE*/
#if (ODBCVER >= 0x0200)
#define SQL_C_SLONG     (SQL_C_LONG+SQL_SIGNED_OFFSET)      /* SIGNED INTEGER     */
#define SQL_C_SSHORT    (SQL_C_SHORT+SQL_SIGNED_OFFSET)     /* SIGNED SMALLINT    */
#define SQL_C_ULONG     (SQL_C_LONG+SQL_UNSIGNED_OFFSET)    /* UNSIGNED INTEGER   */
#define SQL_C_USHORT    (SQL_C_SHORT+SQL_UNSIGNED_OFFSET)   /* UNSIGNED SMALLINT     */
#define SQL_C_STINYINT  (SQL_TINYINT+SQL_SIGNED_OFFSET) /* SIGNED TINYINT   */
#define SQL_C_UTINYINT  (SQL_TINYINT+SQL_UNSIGNED_OFFSET) /* UNSIGNED TINYINT   */
#define SQL_C_BOOKMARK  SQL_C_ULONG                     /* BOOKMARK         */
#endif  /* ODBCVER >= 0x0200 */

/* specify no type conversion for SQLBindCol */
#define SQL_NO_CONVERT    99
/* SAG CLI compliant name */
#define SQL_C_DEFAULT     SQL_NO_CONVERT

/* SQLTransact Options */
#define SQL_COMMIT                 0
#define SQL_ROLLBACK               1

/* SQLError option */
#define SQL_NULL_HENV              NULL /* Note that in ODBC these are 0 */
#define SQL_NULL_HDBC              NULL
#define SQL_NULL_HSTMT             NULL
           
RETCODE SS_EXPORT SQLAllocEnv(
        HENV* phenv);

RETCODE SS_EXPORT SQLFreeEnv(
        HENV henv);

RETCODE SS_EXPORT SQLAllocConnect(
        HENV henv,
        HDBC* phdbc);

RETCODE SS_EXPORT SQLFreeConnect(
        HDBC hdbc);

RETCODE SS_EXPORT SQLConnect(
        HDBC hdbc,
        UCHAR* szDSN,
        SWORD cbDSN,
        UCHAR* szUID,
        SWORD cbUID,
        UCHAR* szAuthStr,
        SWORD cbAuthStr);

RETCODE SS_EXPORT SQLDisconnect(
        HDBC hdbc);

RETCODE SS_EXPORT SQLAllocStmt(
        HDBC hdbc,
        HSTMT* phstmt);

RETCODE SS_EXPORT SQLFreeStmt(
        HSTMT hstmt,
        UWORD fOption);

RETCODE SS_EXPORT SQLPrepare(
        HSTMT hstmt,
        UCHAR* szSqlStr,
        SDWORD cbSqlStr);

RETCODE SS_EXPORT SQLSetParamValue(
        HSTMT hstmt,
        UWORD ipar,
        SWORD fCType,
        SWORD fSqlType,
        UDWORD cbColDef,
        SWORD ibScale,
        PTR rgbValue,
        SDWORD* pcbValue);

RETCODE SS_EXPORT SQLSetCursorName(
        HSTMT hstmt,
        UCHAR* szCursor,
        SWORD cbCursor);

RETCODE SS_EXPORT SQLGetCursorName(
        HSTMT hstmt,
        UCHAR* szCursor,
        SWORD cbCursorMax,
        SWORD* pcbCursor);

RETCODE SS_EXPORT SQLExecute(
        HSTMT hstmt);

RETCODE SS_EXPORT SQLExecDirect(
        HSTMT hstmt,
        UCHAR* szSqlStr,
        SDWORD cbSqlStr);

RETCODE SS_EXPORT SQLNumResultCols(
        HSTMT hstmt,
        SWORD* pccol);

RETCODE SS_EXPORT SQLDescribeCol(
        HSTMT hstmt,
        UWORD icol,
        UCHAR* szColName,
        SWORD cbColNameMax,
        SWORD* pcbColName,
        SWORD* pfSqlType,
        UDWORD* pcbColDef,
        SWORD* pibScale,
        SWORD* pfNullable);

RETCODE SS_EXPORT SQLFetch(
        HSTMT hstmt);

RETCODE SS_EXPORT SQLGetData(
        HSTMT hstmt,
        UWORD icol,
        SWORD fCType,
        PTR rgbValue,
        SDWORD cbValueMax,
        SDWORD* pcbValue);

RETCODE SS_EXPORT SQLGetCol(
        HSTMT hstmt,
        UWORD icol,
        SWORD fCType,
        PTR rgbValue,
        SDWORD cbValueMax,
        SDWORD* pcbValue);

RETCODE SS_EXPORT SQLRowCount(
        HSTMT hstmt,
        SDWORD* pcrow);

RETCODE SS_EXPORT SQLTransact(
        HENV henv,
        HDBC hdbc,
        UWORD fType);

RETCODE SS_EXPORT SQLError(
        HENV henv,
        HDBC hdbc,
        HSTMT hstmt,
        UCHAR* szSqlState,
        SDWORD* pfNativeError,
        UCHAR* szErrorMsg,
        SWORD cbErrorMsgMax,
        SWORD* pcbErrorMsg);

#endif /* CLI0LCLI_H */
