/*************************************************************************
**  source       * aclibstandalone.c
**               *
**  description  * Solid Accelerator usage sample
**               *
**               * (C) Copyright Solid Information Technology Ltd 2003
\*************************************************************************/

/* Common system headers */
#ifdef SS_NT
#  include <windows.h>
#endif

#include <stdio.h>
#include <string.h>

#ifdef SS_WCE
char *argv[] = { "saexampl", NULL };
int main(int argc, char **argv);
#  define assert(f) if(!f) MessageBox(NULL, TEXT("ASSERTION FAILURE"), \
                                      TEXT("ASSERTION FAILURE"), MB_OK)

WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine,
               int nShowCmd )
{        
        main(1,argv);
#  undef UNICODE
#  undef _UNICODE
}

#else
#  include <assert.h>
#endif /* SS_WCE */

#include <solidodbc3.h>

/* Solid Accelerator header */
#include "sscapi.h"

/* Solid portability header */
#include "c.h"

#ifdef SS_CRX
#  define MAXCOLS  100
#else
#  define MAXCOLS  1000
#endif

#define COUNT3 10

#define DATALEN  255
#define MAX_NAME_LEN 80

/* Utility macro for error handling */
#define CHECK(r, hstmt)                                                  \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) {  \
            print_error(r, SQL_HANDLE_STMT,hstmt);                       \
        }                                                                \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

#define CHECKDBC(r, hdbc)                                                \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) {  \
            print_error(r, SQL_HANDLE_DBC,hdbc);                         \
        }                                                                \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

#define CHECKENV(r, henv)                                                \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) {  \
            print_error(r, SQL_HANDLE_ENV,henv);                         \
        }                                                                \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

/* Connecting to datasource */
#define Connect(henv, hdbc, hstmt, sname, uname, pwd) do {           \
        SQLRETURN r;                                                 \
        r = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);  \
        CHECKENV(r, henv);                                           \
        r = SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION,               \
                          (void *)SQL_OV_ODBC3, SQL_NTS);            \
        CHECKENV(r, henv);                                           \
        r = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);             \
        CHECKDBC(r, hdbc);                                           \
        r = SQLConnect(hdbc, sname, SQL_NTS, uname, SQL_NTS,         \
                       pwd, SQL_NTS);                                \
        CHECKDBC(r, hdbc);                                           \
        SQLSetConnectAttr(hdbc, SQL_ATTR_AUTOCOMMIT, com,            \
                          SQL_IS_INTEGER);                           \
        r = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);           \
        CHECKDBC(r, hdbc);                                           \
} while(0)

/* Global variables */
SQLHDBC    hdbc;
SQLHENV    henv;
SQLHSTMT   hstmt;
SQLCHAR   *username = (SQLCHAR*) "dba";
SQLCHAR   *password = (SQLCHAR*) "dba";
SQLPOINTER com = SQL_AUTOCOMMIT_OFF;
SscServerT hh;

/*=============================================================
  Function Name   : print_error()
  Purpose                 : to print the error messages
  Parameters              :
          rc              : Return code
          handle_type     : Handle type identifier
          handle          : handle
  Returns                 : none
==============================================================*/
static void print_error(SQLRETURN rc, SQLSMALLINT handle_type,
                        SQLHANDLE handle)
{
        SQLCHAR      SQLstate[MAX_NAME_LEN];
        SQLINTEGER   errorcode;
        SQLCHAR      errormsg[SQL_MAX_MESSAGE_LENGTH];
        SQLSMALLINT  msglen;
        SQLRETURN    r;
        
        printf("Error (%d) encountered!\n", rc);
        
        /* Print more error info, if available */
        r = SQLGetDiagRec(handle_type, handle, 1, (SQLCHAR *)SQLstate,
                          (SQLINTEGER *)&errorcode, (SQLCHAR *)errormsg,
                          SQL_MAX_MESSAGE_LENGTH - 1, &msglen);
        if (r != SQL_NO_DATA_FOUND) {
            printf("%s\n", errormsg);
        }
}


/*=============================================================
  Function Name   : commit()
  Purpose                 : to commit the transactions
  Parameters              :
          handleType      : Handle type identifier
          handle          : handle
  Returns                 : none
==============================================================*/

void commit(SQLSMALLINT handleType, SQLHANDLE handle)
{
        SQLRETURN r;
        r = SQLEndTran(handleType, handle, SQL_COMMIT);
        CHECKDBC(r, handle);
}


/*=============================================================
  Function Name   : sql_execute()
  Purpose                 : to execute the query
  Parameters              :
          hstmt           : Statement Handle
          sql             : query to be executed
          cbsql           : Length of query
  Returns                 : none
==============================================================*/
static int sql_execute(SQLHSTMT hstmt, SQLCHAR *sql, SQLINTEGER cbsql)
{
        SQLRETURN     rc, rc1, rc2;
        SQLSMALLINT   ColCount = 0;
        SQLCHAR       ColName[COUNT3] = { 0 };
        SQLLEN        Len[MAXCOLS]    = { 0 };
        SQLCHAR       Data[MAXCOLS][DATALEN] = { { 0 } };
        SQLINTEGER    i = 0, RowCount = 0;
        SQLSMALLINT   iCol = 0;
        
        assert(sql);
        
        printf("%s\n", sql);    
        
        rc = SQLExecDirect(hstmt, sql, cbsql);
        
        if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO) {

            rc1 = SQLNumResultCols(hstmt, &ColCount);

            if (ColCount > 0) {
                for (i = 0; i < ColCount; i++) {
                    iCol = i + 1;
                    rc1 = SQLDescribeCol(hstmt, iCol, ColName, sizeof(ColName),
                                         NULL, NULL, NULL, NULL, NULL);
                    
                    assert(rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO);
                    
                    printf("%s\t", ColName);
                    
                    rc1 = SQLBindCol(hstmt, iCol, SQL_C_CHAR, 
                                     Data[iCol-1], sizeof(Data[iCol-1]),
                                     &Len[iCol-1]);
                }
                
                printf("%s","\n");
                
                rc2 = SQLFetch(hstmt);          
                while (rc2 != SQL_NO_DATA_FOUND
                       && rc2 != SQL_ERROR) {
                
                    RowCount++;
                    for (i = 0; i < ColCount; i++) {
                        if (Len[i] == SQL_NULL_DATA) {
                            strcpy((char*) Data[i], "NULL");
                        }
                        printf("%s\t",Data[i]);
                    }
                    
                    printf("%s", "\n");
                    rc2 = SQLFetch(hstmt);
                }
                
                SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
                SQLFreeStmt(hstmt, SQL_UNBIND);
                SQLFreeStmt(hstmt, SQL_CLOSE);
            }
        }

        return(rc);
}


/*=============================================================
  Function Name   : Disconnect()
  Purpose                 : to free the handles
  Parameters              :
          henv            : Environment handle
          hdbc            : Connection Handle
          hstmt           : Statement Handle
  Returns                 : Boolean
==============================================================*/
void Disconnect(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt)
{
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
        SQLDisconnect(hdbc);
        SQLFreeHandle(SQL_HANDLE_DBC, hdbc);            
        SQLFreeHandle(SQL_HANDLE_ENV, henv);
}

/*=============================================================
  Function Name   : acc_start_server()
  Purpose                 : to start the server
  Parameters              :
          a1              : Testcase name
  Returns                 : Boolean
==============================================================*/
static bool acc_start_server(char *a1)
{
        int g_argc, rc;
        char* g_argv[4];
        
        g_argc = 4;
        g_argv[0] = a1; 
        g_argv[1] = "-Udba";
        g_argv[2] = "-Pdba";            
        g_argv[3] = "-Cdba";
        
        rc = SSCStartServer(g_argc, g_argv, &hh, SSC_STATE_OPEN);
        assert(rc == SSC_SUCCESS);

        printf("%s\n", "Server started ...");
        
        return (TRUE);
}

/*=============================================================
  Function Name   : acc_stop_server()
  Purpose                 : to stop the server
  Parameters              : None
  Returns                 : Boolean
==============================================================*/
static bool acc_stop_server(void)
{
        int rc;
        rc = SSCStopServer(hh, TRUE);
        
        return (TRUE);
}

/*##**********************************************************************\
 *
 *              main
 *
 * SQL query execution program main function.
 *
 * Return value - give :
 *       0      = Success return
 *      -1      = Error occurred
 */

#ifdef main
#  undef main
#endif

#if defined(SS_NT)
#  ifndef SS_CDECL
#    define SS_CDECL __cdecl
#  endif
#elif defined(SS_VXW)
#  define SS_CDECL
#  define main embed_main
#else
#  define SS_CDECL
#endif

int SS_CDECL main(void)
{
        SQLRETURN     r;
        bool          b1 = TRUE;
        
        /* Start the replica server using SSCStartServer() */           
        b1 = acc_start_server("");
        assert(b1 == TRUE);
        
        /* Connect to the server */
        Connect(henv, hdbc, hstmt, (SQLCHAR*) "", username, password);
        
        /* Create table */
        r = sql_execute(hstmt, 
                        (SQLCHAR*) "CREATE TABLE TestTable (i INTEGER PRIMARY KEY, y VARCHAR)",
                        SQL_NTS);
        CHECK(r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDBC(r, hdbc);
        
        /* Insert records*/
        r = sql_execute(hstmt,
                        (SQLCHAR*) "INSERT INTO TestTable VALUES (1, 'Row1')", 
                        SQL_NTS);
        CHECK(r, hstmt);
        
        r = sql_execute(hstmt,
                        (SQLCHAR*) "INSERT INTO TestTable VALUES (2, 'Row2')", 
                        SQL_NTS);
        CHECK(r, hstmt);
        
        r = sql_execute(hstmt,
                        (SQLCHAR*) "INSERT INTO TestTable VALUES (3, 'Row3')", 
                        SQL_NTS);
        CHECK(r, hstmt);
        
        r = sql_execute(hstmt,
                        (SQLCHAR*) "INSERT INTO TestTable VALUES (4, 'Row4')", 
                        SQL_NTS);
        CHECK(r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDBC(r, hdbc);


        /* Select records from */
        r = sql_execute(hstmt, (SQLCHAR*) "SELECT * FROM TestTable", 
                        SQL_NTS);
        CHECK(r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDBC(r, hdbc);
        
        /* Disconnect from the server */
        Disconnect(henv, hdbc, hstmt);
                
        /* Shutdown of replica server */
        b1 = acc_stop_server();
        assert(b1 == TRUE);

        return 0;      
}

