/*************************************************************************
**  source       * acsNet.c
**               *
**  description  * Solid Accelerator usage sample
**               *
**               * Embedded SQL statements translated to dynamic CLI
**               * function calls
**               *
**               * (C) Copyright Solid Information Technology Ltd 2001
\*************************************************************************/

#ifdef SS_NT
#  include <windows.h>
#endif

#include <stdio.h>
#include <string.h>

#ifdef SS_WCE
char *argv[] = { "saexampl", NULL };
int main(int argc, char **argv);
#  define assert(f) if(!f) MessageBox(NULL, TEXT("ASSERTION FAILURE"), \
                                      TEXT("ASSERTION FAILURE"), MB_OK)

WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
               LPWSTR lpCmdLine, int nShowCmd )
{
        main(1,argv);        
#  undef UNICODE
#  undef _UNICODE
}
#else
#  include <assert.h>
#endif /* SS_WCE */

#include <solidodbc3.h>

/* Solid Accelerator header*/
#include "sscapi.h"

/* Solid portabilty header */
#include "c.h"

/* Constant definitions */
#define COUNT1  256
#define COUNT3  512
#define COUNT4  1024

#ifdef SS_CRX
#  define MAXCOLS  100
#else
#  define MAXCOLS  1000
#endif

#define DATALEN  255
#define MAX_NAME_LEN 80

/* Utility macro for error handling */
#define CHECK(r, hstmt)  \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) \
            print_error(r, SQL_HANDLE_STMT,hstmt); \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

#define CHECKDbc(r, hdbc)  \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) \
            print_error(r, SQL_HANDLE_DBC,hdbc); \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

#define CHECKEnv(r, henv)  \
        if ( ((r) != SQL_SUCCESS) && ((r) != SQL_SUCCESS_WITH_INFO) ) \
            print_error(r, SQL_HANDLE_ENV,henv); \
        assert( ((r) == SQL_SUCCESS) || ((r) == SQL_SUCCESS_WITH_INFO) )

/* Connecting to datasource  */
#define Connect(henv, hdbc, hstmt, sname, uname, pwd) do {\
		SQLRETURN r;\
		r = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);\
		CHECKEnv(r, henv);\
		r = SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION, (void *)SQL_OV_ODBC3, SQL_NTS);\
		CHECKEnv(r, henv);\
		r = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);\
		CHECKDbc(r, hdbc);\
		r = SQLConnect(hdbc, sname, SQL_NTS, uname, SQL_NTS, pwd, SQL_NTS);\
		CHECKDbc(r, hdbc);\
		SQLSetConnectAttr(hdbc, SQL_ATTR_AUTOCOMMIT, com, SQL_IS_INTEGER);\
		r = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);\
		CHECKDbc(r, hdbc);\
} while(0)

/* Global variables */
SQLHDBC    hdbcr, hdbcm;
SQLHENV    henvr, henvm;
SQLHSTMT   hstmtr, hstmtm;
SQLCHAR *  mastername  = "MAS1";
SQLCHAR *  replicaname = "REP1";
SQLCHAR *  masterat = "TCP 2315";  /* network name of master to which
                                    * messages are forwarded */
SQLCHAR *  masterlistenat = "TCP 2315"; /* master dsn */ 
SQLCHAR *  replicaat = ""; /* replica dsn */
SQLCHAR *  username = "dba";
SQLCHAR *  password = "dba";
SQLPOINTER com = SQL_AUTOCOMMIT_OFF;
SscServerT hh;

/*=============================================================
Function Name	: print_error()
Purpose			: to print the error messages
Parameters		:
	rc			: Return code
	handle_type	: Handle type identifier
	handle		: handle
Returns			: none
==============================================================*/
static void print_error(SQLRETURN rc, SQLSMALLINT handle_type,
                        SQLHANDLE handle)
{
       SQLCHAR      SQLstate[MAX_NAME_LEN];
       SQLINTEGER   errorcode;
       SQLCHAR      errormsg[SQL_MAX_MESSAGE_LENGTH];
       SQLSMALLINT  msglen;
       SQLRETURN    r;
       
       printf("Error (%d) encountered!\n", rc);
       
       /* Print more error info, if available */
       r = SQLGetDiagRec(handle_type,handle,1,(SQLCHAR *)SQLstate,
                         (SQLINTEGER *)&errorcode, (SQLCHAR *)errormsg,
                         SQL_MAX_MESSAGE_LENGTH - 1, &msglen);
       if (r != SQL_NO_DATA_FOUND) {
           printf("%s\n", errormsg);
       }
}


/*=============================================================
Function Name	: commit()
Purpose			: to commit the transactions
Parameters		:
	handleType	: Handle type identifier
	handle		: handle
Returns			: none
==============================================================*/

void commit(SQLSMALLINT handleType, SQLHANDLE handle)
{
        SQLRETURN r;
        r = SQLEndTran(handleType, handle, SQL_COMMIT);
        CHECKDbc( r, handle);
}


/*=============================================================
Function Name	: sql_execute()
Purpose			: to execute the query
Parameters		:
	hstmt		: Statement Handle
	sql			: query to be executed
	cbsql		: Length of query
Returns			: none
==============================================================*/
static int sql_execute(SQLHSTMT hstmt, SQLCHAR *sql, SQLINTEGER cbsql)
{
        SQLRETURN	rc,rc1,rc2;
        SQLSMALLINT	ColCount=0;
        SQLCHAR		ColName[COUNT3]={0};
        SQLLEN      Len[MAXCOLS]={0};
        SQLCHAR		Data[MAXCOLS][DATALEN]={0};
        SQLINTEGER 	i=0,RowCount=0;
        SQLSMALLINT	iCol = 0;
        
        assert(sql);
        
        printf("%s\n", sql);	
        
        rc = SQLExecDirect(hstmt, sql, cbsql);
        
        if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO) {
            rc1 = SQLNumResultCols(hstmt, &ColCount);

            if (ColCount > 0) {
                for (i = 0; i < ColCount; i++) {
                    iCol = i+1;
                    rc1 = SQLDescribeCol(hstmt, iCol, ColName, sizeof(ColName),
                                         NULL, NULL, NULL, NULL, NULL);
                    
                    assert(rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO);
                    
                    printf("%s\t", ColName);
                    
                    rc1 = SQLBindCol(hstmt, iCol, SQL_C_CHAR, 
                                     Data[iCol-1],
                                     sizeof(Data[iCol-1]),
                                     &Len[iCol-1]);
                }
                
                printf("\n");
                
                rc2 = SQLFetch(hstmt);		
                while (rc2 != SQL_NO_DATA_FOUND
                       && rc2 != SQL_ERROR) {

                    RowCount++;
                    for (i = 0; i < ColCount; i++) {
                        if (Len[i] == SQL_NULL_DATA) {
                            strcpy(Data[i],"NULL");
                        }
                        printf("%s\t",Data[i]);
                    }
                    
                    printf("\n");
                    rc2 = SQLFetch(hstmt);
                }
                
                SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
                SQLFreeStmt(hstmt, SQL_UNBIND);
                SQLFreeStmt(hstmt, SQL_CLOSE);
            }
        }
        return(rc);
}


/*=============================================================
Function Name	: regmaster()
Purpose			: to configure database as Master
Parameters		:
	henv		: Environment Handle
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	mastername	: Node name of Master
Returns			: Boolean
==============================================================*/
static bool regmaster(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt,
                      char* mastername)
{
        SQLRETURN r;
        char qry[COUNT1];
        
        sprintf(qry, "%s %s", "SET SYNC NODE ", mastername);
        r = sql_execute(hstmt ,qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        r = sql_execute(hstmt,"SET SYNC MASTER YES",SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return(TRUE);
}

/*=============================================================
Function Name	: regreplica()
Purpose			: to configure database as Replica and to register replica with
						Master database
Parameters		:
	henv		: Environment Handle
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	replicaname	: Node name of Replica
	username	: User name for registeration
	password	: Password of user
	masat		: Listen name of Master
	masname		: Master node name
Returns			: Boolean
==============================================================*/
static bool regreplica(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt,
                       char* replicaname,
                       char * username, char * password, char * masat, 
                       char * masname)
{
        SQLRETURN r;
        char qry[COUNT3];
        
        sprintf(qry,"%s %s", "SET SYNC NODE ", replicaname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);

        r = sql_execute(hstmt,"SET SYNC REPLICA YES", SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "SET SYNC USER ", username,
                " IDENTIFIED BY ",
                password);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s", "MESSAGE msg",
                " BEGIN TO ", mastername);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg",
                " APPEND REGISTER REPLICA ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg", " END ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "MESSAGE msg", " FORWARD TO '",
                masterat, "' TIMEOUT DEFAULT ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s", "MESSAGE msg1", " BEGIN ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg1", " APPEND SYNC_CONFIG('%') ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg1", " END");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "MESSAGE msg1", " FORWARD TO '",
                masat, "' TIMEOUT DEFAULT");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        r = sql_execute(hstmt, "SET SYNC USER NONE", SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return (TRUE);
}

/*=============================================================
Function Name	: createpub()
Purpose			: to create a publication
Parameters		:
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	pname		: Name of Publication
	rtable		: Replica table name
	mtable		: Master table name
Returns			: Boolean
==============================================================*/
static bool createpub(SQLHDBC hdbc, SQLHSTMT hstmt, char* pname,
					  char * rtable, char * mtable)
{
        SQLRETURN r;
        char qry[COUNT4];
        
        sprintf(qry, "%s %s %s %s %s %s %s", " \"CREATE PUBLICATION ", pname, 
                " BEGIN  RESULT SET FOR ", rtable, " BEGIN  SELECT * FROM ",
                mtable, "; END END \"");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK(r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc(r, hdbc);

        return(TRUE);
}

/*=============================================================
Function Name	: regpub()
Purpose			: to register the publication
Parameters		:
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	pname		: Name of Publication
	masat		: Listening name of Master
	masname		: Master node name
Returns			: Boolean
==============================================================*/
static bool regpub(SQLHDBC hdbc, SQLHSTMT hstmt, char* pname,
                   char * masat, char * masname)
{	
        SQLRETURN r;
        char qry[COUNT4];
        
        sprintf(qry, "%s %s %s", "MESSAGE msg2", " BEGIN TO ", masname );
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry,"%s %s %s",  "MESSAGE msg2", 
                " APPEND REGISTER PUBLICATION ", pname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry,"%s %s",  "MESSAGE msg2", " END");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        
        sprintf(qry,"%s %s %s %s",  "MESSAGE msg2", " FORWARD TO '", 
                masat, "' TIMEOUT DEFAULT");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return(TRUE);
}

/*=============================================================
Function Name	: propagate()
Purpose			: to propagate records to master from replica
Parameters		:
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	masname		: Master node name
	masat		: Listening name of Master
Returns			: Boolean
==============================================================*/
static bool propagate(SQLHDBC hdbc, SQLHSTMT hstmt, char * masname, 
					  char * masat)
{
        SQLRETURN r;
        char qry[COUNT3];

        sprintf(qry, "%s %s %s", "MESSAGE msg4", " BEGIN TO ", 
                masname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg4", 
                " APPEND PROPAGATE TRANSACTIONS ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg4", " END");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "MESSAGE msg4", " FORWARD TO '",
                masat, " ' TIMEOUT FOREVER ");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return(TRUE);
}

/*=============================================================
Function Name	: subscribe()
Purpose			: to subscribe records from master
Parameters		:
	henv		: Environment Handle
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	pname		: Publication name
	masname		: Master node name
	masat		: Listening name of Master
Returns			: Boolean
==============================================================*/
static bool subscribe(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, char* pname, 
					  char * masname, char * masat)
{	
        SQLRETURN r;
        char qry[COUNT3];
        
        sprintf(qry, "%s %s %s", "MESSAGE msg3", " BEGIN TO ", 
                masname); 
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s %s", "MESSAGE msg3", 
                " APPEND SUBSCRIBE ", pname); 
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg3", " END "); 
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "MESSAGE msg3", 
                " FORWARD TO '", masat, "' TIMEOUT DEFAULT"); 
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return (TRUE);
}

/*=============================================================
Function Name	: unregpub()
Purpose			: to unregister the publication
Parameters		:
	henv		: Environment Handle
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	pname		: Publication name
	masname		: Master node name
	masat		: Listening name of Master
Returns			: Boolean
==============================================================*/
static bool unregpub(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, char* pname, 
					 char * masname, char * masat)
{
        SQLRETURN r;
        char qry[COUNT4];

        sprintf(qry, "%s %s %s", "MESSAGE msg5", " BEGIN TO ", 
                masname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s %s", "MESSAGE msg5", 
                " APPEND UNREGISTER PUBLICATION ", pname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        
        sprintf(qry, "%s %s", "MESSAGE msg5", " END"); 
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s %s %s", "MESSAGE msg5", " FORWARD TO '",
                masat, "' TIMEOUT FOREVER"); 		
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);			
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return(TRUE);
}

/*=============================================================
Function Name	: dropsynctable()
Purpose			: to drop a table having synchistory
Parameters		:
	hdbc		: Connection Handle
	hstmt		: Statement Handle
	tname		: Table name
Returns			: Boolean
==============================================================*/
static bool dropsynctable(SQLHDBC hdbc, SQLHSTMT hstmt, char* tname)
{
        SQLRETURN r;
        SQLCHAR qry[COUNT3];
        
        sprintf(qry, "%s %s %s", "ALTER TABLE ", tname, " SET NOSYNCHISTORY");
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);			
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        sprintf(qry, "%s %s", "DROP TABLE ", tname);
        r = sql_execute(hstmt, qry, SQL_NTS);
        CHECK( r, hstmt);			
        commit(SQL_HANDLE_DBC, hdbc);
        CHECKDbc( r, hdbc);
        
        return(TRUE);
}

/*=============================================================
Function Name	: repdrop()
Purpose			: to drop replica database from master
Parameters		:
	mhenv		: Environment handle
	mhdbc		: Connection Handle
	mhstmt		: Statement Handle
	repname		: Replica name
Returns			: Boolean
==============================================================*/
static bool repdrop(SQLHENV mhenv, SQLHDBC mhdbc, SQLHSTMT mhstmt,
                    char* repname)
{
        SQLRETURN r;
        char qry[COUNT3];
        
        sprintf(qry, "%s %s", "DROP REPLICA ", repname);
        r = sql_execute(mhstmt, qry, SQL_NTS);
        CHECK( r, mhstmt);			
        commit(SQL_HANDLE_DBC, mhdbc);
        CHECKDbc( r, mhdbc);
        
        r = sql_execute(mhstmt, "SET SYNC MASTER NO", SQL_NTS);
        CHECK( r, mhstmt);			
        commit(SQL_HANDLE_DBC, mhdbc);
        CHECKDbc( r, mhdbc);

        return(TRUE);
}

/*=============================================================
Function Name	: masdrop()
Purpose			: to drop master database from replica
Parameters		:
	rhenv		: Environment handle
	rhdbc		: Connection Handle
	rhstmt		: Statement Handle
	masname		: Master name
Returns			: Boolean
==============================================================*/
static bool masdrop(SQLHENV rhenv, SQLHDBC rhdbc, SQLHSTMT rhstmt, char* masname)
{
        SQLRETURN r;
        char qry[COUNT3];

        sprintf(qry, "%s %s", "DROP MASTER ", masname);
        r = sql_execute(rhstmt, qry, SQL_NTS);
        CHECK( r, rhstmt);			
        commit(SQL_HANDLE_DBC, rhdbc);
        CHECKDbc( r, rhdbc);

        r = sql_execute(rhstmt, "SET SYNC REPLICA NO", SQL_NTS);
        CHECK( r, rhstmt);			
        commit(SQL_HANDLE_DBC, rhdbc);
        CHECKDbc( r, rhdbc);

        return(TRUE);
}

/*=============================================================
Function Name	: Disconnect()
Purpose			: to free the handles
Parameters		:
	henv		: Environment handle
	hdbc		: Connection Handle
	hstmt		: Statement Handle
Returns			: Boolean
==============================================================*/
void Disconnect(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt)
{
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
        SQLDisconnect(hdbc);
        SQLFreeHandle(SQL_HANDLE_DBC, hdbc);		
        SQLFreeHandle(SQL_HANDLE_ENV, henv);
}

/*=============================================================
Function Name	: acc_start_server()
Purpose			: to start the server
Parameters		:
	a1			: Testcase name
Returns			: Boolean
==============================================================*/
static bool acc_start_server(char * a1)
{
        int g_argc, rc;
        char * g_argv[4];

        g_argc = 4;
        g_argv[0] = a1;	
        g_argv[1] = "-Udba";
        g_argv[2] = "-Pdba";		
        g_argv[3] = "-Cdba";
        
        rc = SSCStartServer(g_argc, g_argv, &hh, SSC_STATE_OPEN);
        assert(rc == SSC_SUCCESS);

        printf("%s\n", "Replica Server started .....");

        return (TRUE);
}

/*=============================================================
Function Name	: acc_stop_server()
Purpose			: to stop the server
Parameters		: None
Returns			: Boolean
==============================================================*/
static bool acc_stop_server()
{
        int rc;
        rc = SSCStopServer(hh, TRUE);
        
        return (TRUE);
}

/*##**********************************************************************\
 *
 *              main
 *
 * SQL query execution program main function.
 *
 * Return value - give :
 *       0      = Success return
 *      -1      = Error occurred
 */

#ifdef main
#  undef main
#endif

#if defined(SS_NT)
#  ifndef SS_CDECL
#    define SS_CDECL __cdecl
#  endif
#elif defined(SS_VXW)
#  define SS_CDECL
#  define main embed_main
#else
#  define SS_CDECL
#endif

int SS_CDECL main(int argc, char * argv[])
{
        SQLRETURN	r;
        bool		b1 = TRUE;

        /* Start the replica server using SSCStartServer() */		
        b1 = acc_start_server(argv[0]);
        assert(b1 == TRUE);

        /* Connect to the replica server */
        Connect(henvr, hdbcr, hstmtr, replicaat, username, password);

        /* Connect to the master server */
        Connect(henvm, hdbcm, hstmtm, masterlistenat, username, password);

        /* Configure master server as master */
        b1 = regmaster(henvm, hdbcm, hstmtm, mastername);
        assert(b1 == TRUE);

        /* Configure replica server as replica */
        b1 = regreplica(henvr, hdbcr, hstmtr, replicaname, username, password, 
				    masterat, mastername);
        assert(b1 == TRUE);

        
        /* Create table and set sync history at replica*/
        r = sql_execute(hstmtr, 
                        "CREATE TABLE sfsnet_tab0001 (x INTEGER PRIMARY KEY, y VARCHAR)", SQL_NTS);
        CHECK( r, hstmtr);
        commit(SQL_HANDLE_DBC, hdbcr);
        CHECKDbc( r, hdbcr);
        
        r = sql_execute(hstmtr, 
                        "ALTER TABLE sfsnet_tab0001 SET SYNCHISTORY", SQL_NTS);
        CHECK( r, hstmtr);
        commit(SQL_HANDLE_DBC, hdbcr);
        CHECKDbc( r, hdbcr);
        
        /* Create table and set sync history at master */
        r = sql_execute(hstmtm, 
                        "CREATE TABLE sfsnet_tab0001 (x INTEGER PRIMARY KEY, y VARCHAR)", SQL_NTS);
        CHECK( r, hstmtm);
        commit(SQL_HANDLE_DBC, hdbcm);
        CHECKDbc( r, hdbcm);
        
        r = sql_execute(hstmtm, 
                        "ALTER TABLE sfsnet_tab0001 SET SYNCHISTORY", SQL_NTS);
        CHECK( r, hstmtm);
        commit(SQL_HANDLE_DBC, hdbcm);
        CHECKDbc( r, hdbcm);	
        
        /* Create publication */
        b1 = createpub(hdbcm, hstmtm, "sfsnet_pub0001", "sfsnet_tab0001",
                       "sfsnet_tab0001");
        assert(b1 == TRUE);
        
        /* Register the publication */
        b1 = regpub(hdbcr, hstmtr, "sfsnet_pub0001", masterat, mastername);
        assert(b1 == TRUE);
        
        /* Save insert records at replica */
        r = sql_execute(hstmtr, "SAVE INSERT INTO sfsnet_tab0001 VALUES (1, 'Solid')", 
                        SQL_NTS);
        CHECK( r, hstmtr);
        r = sql_execute(hstmtr, "SAVE INSERT INTO sfsnet_tab0001 VALUES (2, 'Solid')", 
						SQL_NTS);
        CHECK( r, hstmtr);
        r = sql_execute(hstmtr, "SAVE INSERT INTO sfsnet_tab0001 VALUES (3, 'Solid')", 
						SQL_NTS);
        CHECK( r, hstmtr);
        r = sql_execute(hstmtr, "SAVE INSERT INTO sfsnet_tab0001 VALUES (4, 'Solid')", 
						SQL_NTS);
        CHECK( r, hstmtr);
        r = sql_execute(hstmtr, "SAVE INSERT INTO sfsnet_tab0001 VALUES (5, 'Solid')", 
						SQL_NTS);
        CHECK( r, hstmtr);
        commit(SQL_HANDLE_DBC, hdbcr);
        CHECKDbc( r, hdbcr);
        
        /* Propagate records to master */
        b1 = propagate(hdbcr, hstmtr, mastername, masterat);
        assert(b1 == TRUE);
        
        /* Subscribe records from master */
        b1 = subscribe(henvr, hdbcr, hstmtr, "sfsnet_pub0001", mastername, 
                       masterat);
        assert(b1 == TRUE);
        
        /* Select records from replica table */
        r = sql_execute(hstmtr, "SELECT * FROM sfsnet_tab0001", 
                        SQL_NTS);
        CHECK( r, hstmtr);
        commit(SQL_HANDLE_DBC, hdbcr);
        CHECKDbc( r, hdbcr);
			
        /* Cleaning of database */
        
        b1 = unregpub(henvr, hdbcr, hstmtr, "sfsnet_pub0001", mastername,
                      masterat);
        assert(b1 == TRUE);
        
        r = sql_execute(hstmtm, "DROP PUBLICATION sfsnet_pub0001", 
                        SQL_NTS);
        CHECK( r, hstmtm);
        commit(SQL_HANDLE_DBC, hdbcm);
        CHECKDbc( r, hdbcm);
        
        b1 = dropsynctable(hdbcr, hstmtr, "sfsnet_tab0001");
        assert(b1 == TRUE);
        
        b1 = dropsynctable(hdbcm, hstmtm, "sfsnet_tab0001");
        assert(b1 == TRUE);
        
        b1 = repdrop(henvm, hdbcm, hstmtm, replicaname);
        assert(b1 == TRUE);
        
        b1 = masdrop(henvr, hdbcr, hstmtr, mastername);
        assert(b1 == TRUE);
        
        /* Disconnect from the server */
        Disconnect(henvm, hdbcm, hstmtm);
        Disconnect(henvr, hdbcr, hstmtr);
		
        /* Shutdown of replica server */
        b1 = acc_stop_server();
        assert(b1 == TRUE);
        
        return 0;      
}

/* EOF */
