/* *********************************************************************** */
// Sample_Hsb.java
// (C) Copyright (c) 2005, Solid Information Technology.  
// All Rights Reserved.
// This is a sample Java program that uses the Hot Standby (HSB) feature.  
/* *********************************************************************** */

/*

To run this sample, both the Primary server and the Secondary server 
should be started and running. To start the servers, follow the 
procedure below.

( I ) Do setup steps as follows.
    1. Set the parameters in the Solid.ini file.  
    2. Start the primary server. 
    3. Check for the role and status.  

   (NOTE: By default, the server starts in the Secondary Broken 
   role; hence, following steps are to switch to primary.)
    4. Switch Default Secondary to primary mode.
    5. Since Secondary is not up and running, we can switch 
       the Primary to "Primary Alone" role. 
    6. Verify the status and role once again.

( II ) Creation of SEED database in Primary.
    1) Verify that the secondary database is not yet started.  
    2. Create required database objects in primary, e.g. tables, 
       views, procedures, etc.
    3. Copy the seed database to the Secondary Directory, using 
       'hsb copy <directory name>'.

( III ) Start the secondary.  
    1. Set the parameters in the secondary's Solid.ini file.  
    2. Start the secondary server. 
    3. Check for the role (should be secondary broken) and status.  


The Sample_Hsb program will perform the following actions.
    1. ADMIN COMMAND 'HSB CONNECT': To connect to the Secondary 
       by Primary server.
    2. Create tables and procedure.
    3. ADMIN COMMAND 'HSB ROLE': To verify the Primary, 
       Secondary, or Standalone role of a server.
    4. ADMIN COMMAND 'HSB STATUS': To verify the status of the 
       hsb LOG and servers.

Compile and Run:
    To compile, type javac Sample_Hsb.java
    To Run, type java Sample_Hsb.

*/


import java.sql.*;
import javax.swing.*;
import java.io.*;



public class Sample_Hsb extends Thread
{   
    // connection to primary server (pcon = Primary CONnection)
    Connection pcon = null;
    // connection to secondary server (scon = Secondary CONnection)
    Connection scon = null;
    // SQL Statements sent to Primary or Secondary.
    Statement pstmt = null;    // (pstmt = Primary STateMentT)
    Statement sstmt = null;    // (sstmt = Secondary STateMenT)
    ResultSet rset = null;
    ResultSetMetaData meta = null;   // e.g. column names etc. 
    // The "query" variable holds the SQL statement that we want 
    // to execute, e.g. "INSERT ...".  
    String query = null;
    // This is set to "COMMIT WORK";
    String com = null;

    Sample_Hsb()
    {

        try
        {
            System.out.println("****** JDBC HSB sample application *******");
            // The recommended way to register Drivers is to use the 
            // Class.forName() method.
            java.sql.Driver d = (java.sql.Driver)Class.forName(
                                "solid.jdbc.SolidDriver").newInstance();
            System.out.println("1> Driver registered.");

            // Ask the user for a connect string
            System.out.println("2> Please enter Connect String with format:");
            System.out.println(
                    "jdbc:solid://<host>:<port>/<user name>/<password>\n");
            System.out.println(
                    "For example: 'jdbc:solid://localhost:2315/dba/dba'");
            System.out.print(
                    "3> Please enter connect string for Primary server (default jdbc:solid://localhost:2315/dba/dba):");
            BufferedReader reader = new BufferedReader(
                    new InputStreamReader(System.in));
            // ConnectStr = "CONNECTion STRing", e.g. URL of the server 
            // that we want to connect to.  
            String ConnectStr = reader.readLine();
            if(ConnectStr.equals("")) {
                    ConnectStr="jdbc:solid://localhost:2315/dba/dba";
            }
    
            // Try to connect to Primary server.
            pcon = java.sql.DriverManager.getConnection(ConnectStr);
            System.out.println("SolidDriver connected to Primary.");
            pstmt = pcon.createStatement();

            // Get connect string for Secondary server and try to 
            // connect to the Secondary server.
            System.out.print(
                    "4> Please enter connect string for Secondary server (default jdbc:solid://localhost:2325/dba/dba):");
            ConnectStr = reader.readLine();
            if(ConnectStr.equals("")) {
                    ConnectStr="jdbc:solid://localhost:2325/dba/dba";
            }
            scon = java.sql.DriverManager.getConnection(ConnectStr);
            System.out.println("SolidDriver connected to Secondary.");
            sstmt=scon.createStatement();
        }
        catch(ClassNotFoundException cnf)
        {
            System.out.println(cnf);
            System.exit(1);
        }
        catch(SQLException se)
        {
            System.out.println(se);
            System.exit(1);
        }
        catch(Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
    
        com = "COMMIT WORK" ;
    
        // Connect primary to secondary.
        query = "ADMIN COMMAND 'HSB CONNECT' ";
        updateExecute(query, pstmt, "Primary");

    
        // Create database objects at primary
    
        query = "DROP TABLE tab_hsb1";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "CREATE TABLE tab_hsb1(sno int, name varchar)";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "DROP TABLE tab_hsb2";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "CREATE TABLE tab_hsb2(sno int, name varchar)";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "DROP TABLE tab_hsb3";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "CREATE TABLE tab_hsb3(sno int, name varchar)";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "DROP PROCEDURE hsb_proc";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        // Create a stored procedure on the primary.  
        // This stored procedure merely inserts 100 values into tab_hsb2.
        query = "CREATE PROCEDURE hsb_proc\n" +
                 "BEGIN\n" +
                 "DECLARE SNO INT;\n" +
                 "SET SNO = 0;\n" +
                 "EXEC SQL PREPARE C " + 
                 "INSERT INTO tab_hsb2 VALUES (?,'RAMESH');\n" +
                 "WHILE (SNO < 100) LOOP\n" +
                   "EXEC SQL EXECUTE C USING (SNO);\n" +
                   "SET SNO = SNO+1;\n" +
                 "END LOOP\n" +
                 "END\n";
    
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        // Verify Roles at primary and Secondary.
        query = "ADMIN COMMAND 'HSB ROLE' ";
        updateExecute(query, pstmt, "Primary");
        updateExecute(query, sstmt, "Secondary");
    
        query = "ADMIN COMMAND 'HSB CONNECT' ";
        updateExecute(query,pstmt,"Primary");
    
        // Call procedure at primary
        query = "CALL hsb_proc ";
        updateExecute(query, pstmt, "Primary");
        updateExecute(com, pstmt, "Primary");
    
        query = "ADMIN COMMAND 'HSB STATUS' ";
        updateExecute(query, pstmt, "Primary");
        updateExecute(query, sstmt, "Secondary");
    
        // Select records at primary and secondary
        query = "SELECT count(*) FROM tab_hsb2 ";
        queryExecute(query, pstmt, "Primary");
        queryExecute(query, sstmt, "Secondary");
    
        for (int i = 0; i < 5; i++)
        {
            query = "INSERT INTO tab_hsb1 VALUES (1,'Pekka')";
            updateExecute(query, pstmt, "Primary");
    
        }
        updateExecute(com, pstmt, "Primary");
    
        query = "ADMIN COMMAND 'HSB ROLE' ";
        updateExecute(query, pstmt, "Primary");
        updateExecute(query, sstmt, "Secondary");
    
    
        query = "SELECT count(*) FROM tab_hsb2 ";
        queryExecute(query, pstmt, "Primary");
        queryExecute(query, sstmt, "Secondary");
    
    
        query = "SELECT count(*) FROM tab_hsb1 ";
        queryExecute(query, pstmt, "Primary");
        queryExecute(query, sstmt, "Secondary");
    
    
    }//end of constructor
    
    
    /* Method to execute SQL Query and print it on Console. */
    public void queryExecute(String query, Statement stmt, String s)
    {
        try 
        {
            System.out.println("\n");
            System.out.println(s + ":" + query);
            rset = stmt.executeQuery(query);

            // Display the names of the columns that are in the ResultSet.
            meta = rset.getMetaData();
            int i = meta.getColumnCount();
            for (int j = 1; j <= i; j++)
            {
                System.out.print(meta.getColumnName(j) + "\t");
            } //end of for

            // While there are more records remaining, display the 
            // data that we retrieved.  
            while(rset.next())
            {
                System.out.print("\n");
                for (int k = 1; k <= i; k++)
                {
                    System.out.print(rset.getString(k) + "\t");
                } //end of for
            } //end of while
        }
        catch(SQLException sql)
        {
            System.out.println(sql.toString());
        }
    
    }//end of method

        
        /* Method to execute SQL Update and print it on Console. */
    public void updateExecute(String update, Statement stmt, String s)
    {
            boolean rowcount = false;
            try 
            {
                System.out.println("\n");
                System.out.println(s + ":" + update);
                rowcount = stmt.execute(update);
            }
            catch(SQLException sql)
            {
                System.out.println(sql.toString());
            }
            
    }//end of method
        
    public static void main(String []arg)
    {
        new Sample_Hsb();
    }

}

// End of Sample_Hsb.java

