/*
 *  SolidTestClient.java
 *
 *  Example: connection to SolidTest EJB
 *  
 *  14.4.2004
 *
 */
package testclient;

import solidtest.server.SolidTableNames;
import solidtest.server.SolidTest;
import solidtest.server.SolidTestHome;
import java.rmi.*;
import javax.naming.*;
import javax.rmi.*;
import java.io.*;
import java.util.Hashtable;

public class SolidTestClient {

	private static final String _JNDINAME = "SolidTest";
	
	public static void main(String[] args) {

		System.out.println("---------------------------------");
		System.out.println(" SolidTestClient starting up ...");
		System.out.println("---------------------------------");
		System.out.println();
		
		try {
            
            Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY,
                    "org.jnp.interfaces.NamingContextFactory");
            env.put(Context.PROVIDER_URL, "localhost:1099");
            env.put("java.naming.factory.url.pkgs",
                    "org.jboss.naming:org.jnp.interfaces");
                    
			System.out.print("Creating the initial context... ");
			InitialContext initialContext = new InitialContext(env);
			System.out.println("OK.");
			
			System.out.print("Initial context lookup for name '"+_JNDINAME+"' ... ");
			Object nsObject = initialContext.lookup(_JNDINAME);
			System.out.println("OK.");
		
			System.out.print("Narrowing the found object to type SolidTestHome ... ");
			SolidTestHome stHome = (SolidTestHome) PortableRemoteObject.narrow(nsObject, SolidTestHome.class);
			System.out.println("OK.");

			System.out.print("Creating the SolidTest remote object ... ");
			SolidTest st = stHome.create();
			System.out.println("OK.");

			System.out.print("Calling getTableNames() in the remote object ... ");
			SolidTableNames stn = st.getTableNames();
			System.out.println("OK.\n");
			
			System.out.println("Dumping Solid Table Names:");
			System.out.println("--------------------------");
			stn.dumpNames(System.out);
			
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
}
