/***************************************************************************
 *
 * File:           SAMPLE1.C
 * Description:    Sample program for Solid Light Client API
 * Date:           2005
 * 
 * Solid Light Client sample program does the following. 
 *
 * 1. Checks that there are enough input parameters to contain sufficient 
 *    connect information
 * 2. Prepares to connect Solid Server throught Light Client by allocating 
 *    memory for HENV and HDBC objects 
 * 3. Connects to SOLID Server using Light Client Library 
 * 4. Creates a statement for one query, 
 *    'SELECT TABLE_SCHEMA,TABLE_NAME,TABLE_TYPE FROM TABLES' for reading 
 *    data from one of Solid Server's system tables. 
 * 5. Executes the query 
 * 6. Fetches and outputs all the rows of a result set. 
 * 7. Closes the connection gracefully.
 *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>

#include "cli0lcli.h"

#if defined(SS_NT)
#  ifndef SS_CDECL
#    define SS_CDECL __cdecl
#  endif
#else
#  define SS_CDECL
#endif

#if defined(SS_VXW)

#  define main sample1_main
int sample1_main(int argc, char *argv[]);
void sample1(void)
{
        char* args[5];
        args[0] = "sample1";
        args[1] = "127.0.0.1 2315";
        args[2] = "DBA";
        args[3] = "DBA";
        args[4] = NULL;

        sample1_main(4, args);

        return;
}

#endif /* SS_VXW */

int SS_CDECL main(int argc, char *argv[])
{
        HENV henv;      /* pointer to environment object            */ 
        HDBC hdbc;      /* pointer to database connection object    */ 
        RETCODE rc;     /* variable for return code                 */ 
        HSTMT hstmt;    /* pointer to database statement object     */ 
        char buf[255];  /* buffer for data to be obtained from db   */
        char buf2[255]; /* buffer for a printable row to be created */
        int iCount = 0; /* counter for rows to be fetched.          */
        
        /* 1. Checks that there are enough input parameters to contain
         *    sufficient connect information  */        
#ifdef SS_CRX
        char* connection="127.0.0.1 2315";
        /* in CRX environment only username and passwd should be given */
        if (argc != 3) {
            printf("Proper usage uid pwd\n");
            printf("argc %d\n", argc);
            return 1;
        }
        printf("Will connect SOLID Server at \"tcp 127.0.0.1 2315\" with uid '%s' and pwd '%s'.\n", argv[1], argv[2]);
#else       
        if (argc != 4) {
            printf("Proper usage \"connect string\" uid pwd\n");
            printf("argc %d\n", argc);
            return 1;
        }
        printf("Will connect SOLID Server at %s with uid %s and pwd %s.\n",argv[1],argv[2],argv[3]);
        
#endif /* SS_CRX */
                
        /* 2. Prepares to connect SOLID Server throught Light Client
         *    by allocating memory for HENV and HDBC objects
         */
        
        rc = SQLAllocEnv(&henv);
        
        if (SQL_SUCCESS != rc) {
            printf("SQLAllocEnv fails.\n"); 
            return 1; 
        } 
        
        rc = SQLAllocConnect(henv, &hdbc); 
        if (SQL_SUCCESS != rc) {
            printf("SQLAllocConnect fails.\n"); 
            return 1; 
        } 
        
        /* 3. Connects to SOLID Server using Light Client Library */
#ifdef SS_CRX
        rc = SQLConnect(hdbc, (UCHAR*)connection, SQL_NTS, (UCHAR*)argv[1],
                        SQL_NTS, (UCHAR*)argv[2], SQL_NTS); 
#else
        rc = SQLConnect(hdbc, (UCHAR*)argv[1], SQL_NTS,
                        (UCHAR*)argv[2], SQL_NTS, (UCHAR*)argv[3], SQL_NTS); 
#endif /* SS_CRX */
        if (SQL_SUCCESS != rc) {
            printf("SQLConnect failed (%d).\n", rc); 
            return 1;   
        }
        printf("Connect ok.\n");
        
        /* 4. Creates a statement for one query, 'SELECT
         *    TABLE_SCHEMA,TABLE_NAME,TABLE_TYPE FROM TABLES' for
         *    reading data from one of SOLID Server's system
         *    tables.
         */
        
        rc = SQLAllocStmt(hdbc, &hstmt); 
        if (SQL_SUCCESS != rc) { 
            printf("SQLAllocStmt failed. \n"); 
        } 
        
        rc = SQLPrepare(hstmt, (UCHAR*)"SELECT TABLE_SCHEMA,TABLE_NAME,TABLE_TYPE FROM TABLES",SQL_NTS); 
        
        if (SQL_SUCCESS != rc) { 
            printf("SQLPrepare failed.\n"); 
        } else {
            printf("SQLPrepare succeeded.\n");
        }
                
        /* 5. Execute the query */
        rc = SQLExecute(hstmt); 
        if (SQL_SUCCESS != rc) { 
            printf("SQLExecute failed.\n"); 
        } else {
            printf("SQLExecute succeeded.\n");
        }
        
        
        /* 6. Fetches and outputs all the rows of a result set. */
        rc = SQLFetch(hstmt); 
        if ((SQL_SUCCESS != rc) && (SQL_NO_DATA_FOUND != rc)) { 
            printf("SQLFetch returned an unexpected error code: %d\n", rc); 
        } else {
            printf("Starting to fetch data.\n");
        }
        
        while (SQL_NO_DATA_FOUND != rc) { 
            iCount++;
            sprintf(buf2, "Row %d :", iCount);
            
            rc = SQLGetCol(hstmt, 1, SQL_C_CHAR, buf, sizeof(buf), NULL); 
            if (SQL_SUCCESS == rc) {
                strcat(buf2, buf);
                strcat(buf2, ",");
            } else {
                printf("Error in SQL_GetCol(1)\n");
            }
            
            rc = SQLGetCol(hstmt, 2, SQL_C_CHAR, buf, sizeof(buf), NULL); 
            if (SQL_SUCCESS == rc) {
                strcat(buf2, buf);
                strcat(buf2, ",");
            } else {
                printf("Error in SQL_GetCol(2)\n");
            }
            
            rc = SQLGetCol(hstmt, 3, SQL_C_CHAR, buf, sizeof(buf), NULL); 
            if (SQL_SUCCESS == rc) {
                strcat(buf2, buf);
            } else {
                printf("Error in SQL_GetCol(3)\n");
            }
            
            printf("%s\n", buf2);
                        
            rc = SQLFetch(hstmt); 
        } 
        
        rc = SQLFreeStmt(hstmt, SQL_DROP); 
        if ((SQL_SUCCESS != rc)) {
            printf("SQLFreeStmt failed."); 
        } 
        
        /* 7. Close the connection gracefully */
        SQLDisconnect(hdbc); 
        SQLFreeConnect(hdbc); 
        SQLFreeEnv(henv);
        
        printf("Sample program ended successfully.\n");        

        return 0;
}

/* EOF */
