-- COPYRIGHT: Copyright (c) 2002, Solid Information Technology.
-- TITLE: SetUpReplica.SQL
-- PURPOSE: This is part of a sample that shows how to implement  
--     Sync Pull Notify (push Synchronization)
-- NOTES: This script creates a table, registers the replica, and 
--     creates a stored procedure that refreshes to get updated data.  
--     None of this is specific to Sync Pull Notify; each of these 
--     could be done on replicas that refresh from a master without 
--     using Sync Pull Notify.  In fact, most of this script, except 
--     the stored procedure and the CREATE TABLE statement, is 
--     directly copied from the SmartFlow Guide's sample of how 
--     to configure and register *any* replica.  



-- Create the table.
CREATE TABLE customers (
	id INT NOT NULL,
	name VARCHAR(30),
	address VARCHAR(100), 
	PRIMARY KEY (id)
	);

COMMIT WORK;


-- Specify that this node is to be a replica node.
SET SYNC NODE REPLICA1;
COMMIT WORK;
SET SYNC MASTER NO;
SET SYNC REPLICA YES;
COMMIT WORK;

SET SYNC USER DBA IDENTIFIED BY DBA;
COMMIT WORK;


-- Register the replica with the master by sending a registration message.
MESSAGE CFG1 BEGIN;
MESSAGE CFG1 APPEND REGISTER REPLICA;
MESSAGE CFG1 END;
COMMIT WORK;
MESSAGE CFG1 FORWARD TO 'tcp 2315' TIMEOUT FOREVER;
COMMIT WORK;


-- Subscribe user information from the master in a separate message.
MESSAGE CFG2 BEGIN;
MESSAGE CFG2 APPEND SYNC_CONFIG('%');
MESSAGE CFG2 END;
COMMIT WORK;
MESSAGE CFG2 FORWARD TO 'tcp 2315' TIMEOUT FOREVER;
COMMIT WORK;
SET SYNC USER NONE;
COMMIT WORK;


-- Register with the publication.  
MESSAGE REG_PUBLICATION1 BEGIN;
MESSAGE REG_PUBLICATION1 APPEND REGISTER PUBLICATION publication1;
MESSAGE REG_PUBLICATION1 END;
COMMIT WORK;
MESSAGE REG_PUBLICATION1 FORWARD TIMEOUT FOREVER;
COMMIT WORK;


-- This creates a procedure that requests a refresh of the updated data.
"CREATE PROCEDURE refresh_from_publication1
BEGIN
-- The msg_name is a dynamically-generated unique name for the message
-- that we will send to the master.
DECLARE msg_name CHAR(40);
-- The sql_str is a string that contains the command that we want to execute.
DECLARE sql_str VARCHAR;

msg_name := get_unique_string('MSG_REFRESH_P1_') ;

sql_str := 'MESSAGE ' + msg_name + ' BEGIN';
EXEC SQL EXECDIRECT sql_str;

sql_str := 'MESSAGE ' + msg_name + ' APPEND REFRESH publication1';
EXEC SQL EXECDIRECT sql_str;

sql_str := 'MESSAGE ' + msg_name + ' END';
EXEC SQL EXECDIRECT sql_str;

EXEC SQL COMMIT WORK ;

-- 30 second timeout. Change this if you wish.
sql_str := 'MESSAGE ' + msg_name + ' FORWARD TIMEOUT 30';
EXEC SQL EXECDIRECT sql_str;

EXEC SQL COMMIT WORK ;

END";

COMMIT WORK;
