/*************************************************************************
**  source       * saembed1.c
**               * 
**  description  * Solid SA-Interface usage sample:
**               * 
**               * Embedded SQL statements translated to dynamic CLI(sa calls) 
**               * function calls with SOLID Accelerator server
**               * 
**               * (C) Copyright Solid Information Technology Ltd 2005
\*************************************************************************/

/* Common system headers */
#ifdef SS_NT
#  include <windows.h>
#endif

#ifndef FALSE
#  define FALSE 0
#endif

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include <solidodbc3.h>

/* SOLID portability header */
#include "c.h"

/* SOLID SA header */
#include "sa.h"

/* SOLID Accelerator API */
#include "sscapi.h"

/* Constants */
#define MAX_NAME_LEN 80

/* Function prototypes */
static void usage(void);
static void welcome(void);
static void embedded_to_dynamic_SQL_example();
static void t_scon_error(SaConnectT *, int, char *, int);
static void t_scur_error(SaCursorT *, int, char *, int);
static void print_assert_info(char *, int);
static int  fetch_data(SaConnectT *);
static void start_Accelerator_server(void);
static void stop_Accelerator_server(void);

/* Utility macros - very crude error handling */
#define t_con_check(scon,expr) t_scon_error(scon,expr,__FILE__,__LINE__);    
#define t_cur_check(scur,expr) t_scur_error(scur,expr,__FILE__,__LINE__);    

/* Accelerator server catalog, username and password 
 * it assumes that the local ini file contains the 
 * listen name as 'tcp 2315' 
 */
static char*    sf_rundir   = "-crun";
static char*    sf_username	= "-Udba";
static char*    sf_password	= "-Pdba";
static char*	sf_catalog	= "-Cdba";
static char*	server		= "";
static char*	uid			= "dba";
static char*	pwd			= "dba";
ssc_serverhandle_t	  sf_handle;


/*=============================================================
Function Name	: main()
Purpose			: ad hoc SQL query execution program main function.
Parameters		: none
Returns			: '0' on success,-1 on failure
==============================================================*/
#if defined(SS_NT)
#  ifndef SS_CDECL
#    define SS_CDECL __cdecl
#  endif
#elif defined(SS_VXW)
#  define SS_CDECL
#  define main embed_main

void embed(){
        char* args[5];
        args[0] = "embed";

        /* ***** IMPORTANT ****
         * The host address 255.255.255.255 is invalid and you have to
         * edit it according to your network configuration.  It's not
         * possible to connect to localhost 127.0.0.1, so Solid
         * DatabaseEngine has to be run in another machine.
         */
        args[1] = "tcp 255.255.255.255 2315";
        args[2] = "dba";
        args[3] = "dba";
        args[4] = NULL;
        embed_main(4, args);
        return;
}

#else
#  define SS_CDECL
#endif

int SS_CDECL main(int argc, char* argv[])
{
        if (argc > 1) {
            usage();
            return(-1);
        }

        welcome();
        start_Accelerator_server();
        embedded_to_dynamic_SQL_example();
        stop_Accelerator_server();
        return(0);
}

/*#***********************************************************************\
 *
 *      usage
 *
 * Print usage
 *
 */
static void usage(void)
{
        printf("\nUsage:  saembed1 \n\n");
        welcome();
}

/*#***********************************************************************\
 *
 *      welcome
 *
 * Print welcome message
 *
 */
static void welcome(void)
{
        printf("This program automatically executes a set of static SQL\n");
        printf("statements and checks their success through SOLID API(SA) interface\n");
        printf("with Accelerator server ...\n\n");
}

/*#***********************************************************************\
 *
 *      start_Accelerator_server
 *
 * Return value:
 *
 * Limitations:
 *
 * Globals used:
 */
static void start_Accelerator_server(void)
{
        SaRetT	g_argc, rc;
        char    *g_argv[5];
		
        g_argc = 5;
        g_argv[0] = __FILE__;	
        g_argv[1] = sf_rundir;
        g_argv[2] = sf_username;
        g_argv[3] = sf_password;		
        g_argv[4] = sf_catalog;

        printf("\n===== starting Accelerator server =====\n");

        rc = SSCStartServer(g_argc, g_argv, &sf_handle, SSC_STATE_OPEN);
        assert(rc == SSC_SUCCESS);

        printf("\n===== Accelerator server started successfully =====\n");
}

/*#***********************************************************************\
 *
 *      stop_Accelerator_server
 *
 */
static void stop_Accelerator_server(void)
{
        SaRetT rc;
        
        printf("\n====== stopping Accelerator server =====\n");
        rc = SSCStopServer(sf_handle, FALSE);
        assert(rc == SSC_SUCCESS);
}

/*#***********************************************************************\
 *
 *      embedded_to_dynamic_SQL_example
 *
 * Example function that contructs SQL statements within the application.
 * The example comments include equivalent embedded SQL calls for
 * illustrative purposes.
 *
 */
static void embedded_to_dynamic_SQL_example(void)
{
        SaRetT	    rc = 0;
        SaConnectT  *scon = NULL;
        SaCursorT   *scur = NULL;
        int		    id;
        char	    *name;

        /********************************************************/

        printf("Connecting to server '%s' ...\n", server);
        
        /* connect to specified server using SaConnect SA-API */

        scon = SaConnect(server, uid, pwd);
        if (scon == NULL) {
            printf("\nFailed to connect to %s\n", server);
            exit(-1);
        }
        
        /********************************************************/

        printf("Creating table 'NAMEID' ...\n");
        
        /* DROP TABLE NAMEID if it exists
         * CREATE TABLE NAMEID(ID integer, NAME varUCHAR(50));
         * Execute the SQL statement.
         */
        
        SaSQLExecDirect(scon, "DROP TABLE NAMEID");
        
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
        
        rc = SaSQLExecDirect(scon, "CREATE TABLE NAMEID (ID INTEGER, NAME VARCHAR(50))");
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /********************************************************/

        printf("Commit.\n");
        
        /* commit the transaction using SaTransCommit */
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /********************************************************/

        printf("Inserting a row to table 'NAMEID' (500, Babbage) ...\n");
        
        /* - Create the cursor scur on the table 'NAMEID' 
         * - Bind input data using SaCursorColInt for integer,
         *     SacursorColStr for string data				
         * - Open the cursor								
         * - Assign parameter values.						
         * - Execute SaCursorInsert to insert the data
         */
        
        scur = SaCursorCreate(scon,"NAMEID");
        if (scur == NULL) {
            printf("\nTable 'NAMEID' not found \n");
            exit(-1);
        }
        rc = SaCursorColInt(scur, "ID", &id);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorColStr(scur, "NAME", &name);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorOpen(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        id = 500;
        name = (char*)malloc(20);
        strcpy(name, "Babbage");
        
        rc = SaCursorInsert(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        /********************************************************/

        printf("Commit.\n");
        
        /* Commit the insertion. */        
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /* Close cursor */
        SaCursorFree(scur);
        free(name);
        
        /********************************************************/

        /* fetching the data back */
        rc = fetch_data(scon);
        assert(1 == rc);
        
        /********************************************************/

        printf("updating rows in table 'NAMEID' ...\n");
        
        scur = SaCursorCreate(scon,"NAMEID");
        if (scur == NULL) {
            printf("\nTable 'NAMEID' not found \n");
            exit(-1);
        }
        rc = SaCursorColStr(scur,"NAME",&name);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorOpen(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorSearch(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorNext(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        name = "updated";
        rc = SaCursorUpdate(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        printf("1 row updated.\n");
        
        /********************************************************/

        printf("Closing cursor ...\n");
        
        /* Free the cursor */        
        SaCursorFree(scur);
        
        /********************************************************/

        printf("Commit.\n");
        
        /* EXEC SQL COMMIT WORK; */
        
        rc = SaTransCommit(scon);
        t_con_check(scon,rc == SA_RC_SUCC);
        
        /********************************************************/

        /* fetching the data back */
        rc = fetch_data(scon);
        assert(1 == rc);
        
        /********************************************************/

        printf("Deleting all rows from table 'NAMEID' ...\n");
        
        scur = SaCursorCreate(scon,"NAMEID");
        if (scur == NULL) {
            printf("\n Table 'NAMEID' not found \n");
            exit(-1);
        }
        rc = SaCursorOpen(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorSearch(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorNext(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);
        
        rc = SaCursorDelete(scur);
        t_cur_check(scur, rc == SA_RC_SUCC);

        printf("1 row deleted.\n");
        
        /********************************************************/

        printf("Closing cursor ...\n");
        
        /* Free the cursor */
        SaCursorFree(scur);

        /********************************************************/
        printf("Commit.\n");
        
        /* EXEC SQL COMMIT WORK;      */
        
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /********************************************************/

        /* fetching the data back */
        rc = fetch_data(scon);
        assert(0 == rc);
        
        /********************************************************/
        printf("Dropping table 'NAMEID' ...\n");
        
        /* EXEC SQL DROP TABLE NAMEID
         * Execute the SQL statement.
         */
        
        rc = SaSQLExecDirect(scon,"DROP TABLE NAMEID");
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /********************************************************/

        printf("Commit.\n");
        
        /* EXEC SQL COMMIT WORK; */
        
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
        
        /********************************************************/

        printf("Disconnecting from server ...\n");
        
        /* EXEC SQL DISCONNECT;
         * 
         *   Disconnect from the data source. 
         *   Free the connection handle.      
         *   Free the environment handle.
         */
        
        SaDisconnect(scon);       
        
}

/*#***********************************************************************\
 *
 *      t_scon_error
 *
 * Retrieve connection based error message, perform assertion.
 *
 * Parameters:
 *      scon - in
 *          active connection
 *
 *      expr - in
 *          to assert or not to assert.
 *
 *      file - in
 *          name of the file the error is about
 *
 *      line - in
 *          line in filename on which the error happened
 *
 */
static void t_scon_error(SaConnectT* scon, int expr, char* file, int line)
{
        char* errstr;
        int   errcode;
        
        if (SaErrorInfo(scon, &errstr, &errcode)) {
            printf("\n%d:[%d][%s]", line, errcode, errstr);		
        }	
        if (!expr) {
            print_assert_info(file, line);
        }
}

/*#***********************************************************************\
 *
 *      t_scon_error
 *
 * Retrieve cursor based error message, perform assertion.
 *
 * Parameters:
 *      scon - in
 *          active cursor
 *
 *      expr - in
 *          to assert or not to assert.
 *
 *      file - in
 *          name of the file the error is about
 *
 *      line - in
 *          line in filename on which the error happened
 *
 */
static void t_scur_error(SaCursorT* scur, int expr, char* file, int line)
{
        char* errstr;
        int   errcode;
        
        if (SaCursorErrorInfo(scur, &errstr, &errcode)) {
            printf("\n%d:[%d][%s]", line, errcode, errstr);			
        }
        if (!expr) {
            print_assert_info(file, line);
        }
}

/*#***********************************************************************\
 *
 *      print_assert_info
 *
 * Print assertion message
 *
 * Parameters:
 *      file - in
 *          file in which assertion occured
 *
 *      line - in
 *          line on which assertion occured
 *
 */
static void print_assert_info(char *file, int line)
{
        printf("\nSOLID assertion failed in %d@%s\n", line, file);
        exit(0);
}

/*#***********************************************************************\
 *
 *      fetch_data
 *
 * Fetch data from table 'NAMEID'
 *
 * Parameters:
 *      scon - in, use
 *          active connection
 *
 * Return value:
 *      number of rows fetched.
 *
 */
static int fetch_data(SaConnectT *scon)
{
        SaCursorT   *scur = NULL;
        SaRetT      rc = 0, rowcount = 0;
        int	        id;
        char        *name;

        printf("Retrieving data from table 'NAMEID'...\n");

        /* - Create the cursor scur on the table 'NAMEID'
         * - Bind output data using SaCursorColInt for integer,
         *	   SacursorColStr for string data				
         * - Open the cursor								
         * - Start the search operation					
         * - Call SaCursorNext to fetch the first row	
         * - Execute SaCursorInsert to insert the data
         */

        scur = SaCursorCreate(scon, "NAMEID");
        if (scur == NULL) {
            printf("\nTable 'NAMEID' not found \n");
            exit(-1);
        }
        rc = SaCursorColInt(scur, "ID", &id);
        t_cur_check(scur, rc == SA_RC_SUCC);

        rc = SaCursorColStr(scur, "NAME", &name);
        t_cur_check(scur, rc == SA_RC_SUCC);

		rc = SaCursorOpen(scur);
		t_cur_check(scur, rc == SA_RC_SUCC);

        SaCursorSearch(scur);
        
        while (SaCursorNext(scur) == SA_RC_SUCC) {
            rowcount++;
            printf("A row found (%d, %s).\n", id, name);
        }
        rc = SaCursorNext(scur);
        t_cur_check(scur, rc == SA_RC_END);

        SaCursorFree(scur);
        
        rc = SaTransCommit(scon);
        t_con_check(scon, rc == SA_RC_SUCC);
		
        printf("total rows: %d\n", rowcount);

        return(rowcount);
}

/* EOF */
