-- ***********************************************
-- THE SAMPLE ANSWER TO EXERCISE TASK 3
-- 
-- PROCEDURE MAKE_DAILY_ORDER_REPORT
--
-- This procedure returns list of those products, 
-- whose inventory balance has been dropped below 
-- alert_limit during the given day. 
-- ***********************************************
DROP PROCEDURE make_daily_order_report ; 
COMMIT WORK;

"CREATE PROCEDURE make_daily_order_report (report_date date) 
RETURNS (product_id integer, product_name varchar, amount_ordered_today decimal(12,2))
BEGIN 
	DECLARE row_num INTEGER ; 
	DECLARE amount_in_stock DECIMAL(12,2) ; 
	DECLARE alert_limit DECIMAL(12,2) ; 

	EXEC SQL WHENEVER SQLERROR ABORT ; 

	row_num := 0 ; 

	EXEC SQL PREPARE sel_rows
	SELECT 	order_line.product_id, 
		product.product_name, 
		sum(order_line.ordered_amount) sum_amount
	FROM 	order, order_line, product
	WHERE 	order_line.order_id = order.id AND
		order_line.product_id = product.id AND 
		order.order_date = ?
	GROUP BY order_line.product_id, product.product_name, order.order_date 
	ORDER BY order.order_date DESC;

	EXEC SQL PREPARE ins_rows 
		INSERT INTO daily_order_report (report_date, row_num, product_id, 
				product_name, amount_ordered_today)
		VALUES (?,?,?,?,?) ;

	EXEC SQL PREPARE check_product  
		SELECT amount_in_stock, alert_limit
		FROM product 
		WHERE id = ? ; 
	
	EXEC SQL EXECUTE sel_rows 
		USING (report_date) 
		INTO (product_id, product_name, amount_ordered_today) ; 

	EXEC SQL FETCH SEL_ROWS ; 
	WHILE SQLSUCCESS LOOP 
		
		EXEC SQL EXECUTE check_product USING (product_id) INTO (amount_in_stock, alert_limit) ; 
		EXEC SQL FETCH check_product ; 
		EXEC SQL CLOSE check_product ;
		IF amount_in_stock < alert_limit THEN  
			row_num := row_num + 1 ; 
			EXEC SQL EXECUTE ins_rows 
				USING (report_date, row_num, product_id, 
				product_name, amount_ordered_today) ; 
		END IF ; 
		RETURN ROW ; 
		EXEC SQL FETCH sel_rows ; 

	END LOOP ; 
	EXEC SQL COMMIT WORK ; 
	EXEC SQL DROP check_product ; 
	EXEC SQL DROP sel_rows ; 
	EXEC SQL DROP ins_rows ; 

END "; 

COMMIT WORK;

-- Note that in this sample the row number is not incremental 
-- limiting the reports to 1/day.

CALL make_daily_order_report ('1998-12-07') ; 

COMMIT WORK;
